@@ -1,7 +1,7 @@
 package Plack::Middleware::PeriAHS::Respond;
 
-our $DATE = '2014-10-23'; # DATE
-our $VERSION = '0.53'; # VERSION
+our $DATE = '2014-10-24'; # DATE
+our $VERSION = '0.54'; # VERSION
 
 use 5.010;
 use strict;
@@ -68,6 +68,11 @@ sub format_result {
             last;
         }
     }
+
+    # do Riap 1.2 stuffs, but only encode binary result if we're sending as
+    # JSON, because the other formats are binary safe
+    insert_riap_stuffs_to_res($rres, $rreq->{v}, undef, $fmt =~ /json/);
+
     my $ct = $formatter->[1];
 
     my $fres = Perinci::Result::Format::format($rres, $fmt);
@@ -174,7 +179,6 @@ sub call {
                 $rres = $pa->request($rreq->{action} => $rreq->{uri}, $rreq);
             }
         }
-        insert_riap_stuffs_to_res($rres, $rreq->{v});
         $cleanser->clean_in_place($rres);
         $env->{'periahs.finish_action_time'} = [gettimeofday];
 
@@ -208,7 +212,7 @@ Plack::Middleware::PeriAHS::Respond - Send Riap request to Riap server and send
 
 =head1 VERSION
 
-This document describes version 0.53 of Plack::Middleware::PeriAHS::Respond (from Perl distribution Perinci-Access-HTTP-Server), released on 2014-10-23.
+This document describes version 0.54 of Plack::Middleware::PeriAHS::Respond (from Perl distribution Perinci-Access-HTTP-Server), released on 2014-10-24.
 
 =head1 SYNOPSIS
 
